﻿
namespace WP7Host.Web.Services
{
    using System.Data;
    using System.Data.Objects;
    using System.Linq;
    using System.ServiceModel.DomainServices.EntityFramework;
    using System.ServiceModel.DomainServices.Hosting;
    using System.ServiceModel.DomainServices.Server;
    using WP7Host.Web;
    using System.Reactive;
    using System.Reactive.Linq;
    using System;

    [EnableClientAccess()]
    public class WP7DomainService : LinqToEntitiesDomainService<WP7ModelContainer>
    {

        [Query(IsDefault = true)]
        public IQueryable<WP7History> GetWP7HistorySet()
        {
            return this.ObjectContext.WP7HistorySet;
        }

        /// <summary>
        /// 조회를 하면서 동시에 히스토리 데이터도 조회 한다.
        /// 점진적 로드를 위해 id로 정렬
        /// </summary>
        /// <returns></returns>
        [Query(IsDefault = true)]
        public IQueryable<WP7UriList> GetWP7UriListSet()
        {
            var query = (
                            (ObjectQuery<WP7UriList>)
                                from kkk in this.ObjectContext.WP7UriListSet
                                orderby kkk.Id
                                select kkk
                        ).Include("WP7History");
            //return this.ObjectContext.WP7UriListSet;

            //WP7History history = new WP7History
            //{
            //    Code = "02",
            //    Action = "목록조회",
            //    RegDT = DateTime.Now,
            //    UptDT = DateTime.Now,
            //    UseBT = true
            //};
            //this.ObjectContext.WP7HistorySet.AddObject(history);

            return query.AsQueryable();
        }

        public IQbservable<WP7UriList> GetWP7UriListSetQb()
        {
            var query = (
                            (ObjectQuery<WP7UriList>)
                                from kkk in this.ObjectContext.WP7UriListSet
                                orderby kkk.Id
                                select kkk
                        ).Include("WP7History");

            return query.ToQbservable();
        }

        public void InsertWP7UriList(WP7UriList wP7UriList)
        {
            var query = from kkk in this.ObjectContext.WP7UriListSet
                        where kkk.UserName == wP7UriList.UserName
                        select kkk;

            if (query.Count() > 0)
            {
                throw new DomainException(wP7UriList.UserName + "은 이미 등록되어 있는 사용자 이름 입니다.");
            }

            WP7History history = new WP7History
            {
                Code = "01",
                Action = "등록",
                RegDT = DateTime.Now,
                UptDT = DateTime.Now,
                UseBT = true,
                WP7UriList = wP7UriList
            };
            this.ObjectContext.WP7HistorySet.AddObject(history);
            wP7UriList.WP7History.Add(history);

            //System.Diagnostics.Debug.WriteLine("Insert Channel Uri : " + wP7UriList.ChannelUri);
            if ((wP7UriList.EntityState != EntityState.Detached))
            {
                this.ObjectContext.ObjectStateManager.ChangeObjectState(wP7UriList, EntityState.Added);
            }
            else
            {
                this.ObjectContext.WP7UriListSet.AddObject(wP7UriList);
            }

        }

        public void UpdateWP7UriList(WP7UriList currentWP7UriList)
        {
            this.ObjectContext.WP7UriListSet.AttachAsModified(currentWP7UriList, this.ChangeSet.GetOriginal(currentWP7UriList));
        }

        [RequiresAuthentication(ErrorMessage="로그인이 필요한 서비스 입니다.")]
        [RequiresRole("Administrator", ErrorMessage="관리자만 삭제가 가능합니다.")]
        public void DeleteWP7UriList(WP7UriList wP7UriList)
        {
            if ((wP7UriList.EntityState != EntityState.Detached))
            {
                this.ObjectContext.ObjectStateManager.ChangeObjectState(wP7UriList, EntityState.Deleted);
            }
            else
            {
                this.ObjectContext.WP7UriListSet.Attach(wP7UriList);
                this.ObjectContext.WP7UriListSet.DeleteObject(wP7UriList);
            }

            //히스토리 삭제
            this.ObjectContext.WP7HistorySet
                    .Where(p => p.WP7UriListId == wP7UriList.Id)
                    .ToList().ForEach(p =>
                    {
                        this.ObjectContext.WP7HistorySet.Attach(p);
                        this.ObjectContext.WP7HistorySet.DeleteObject(p);
                    });

        }
    }
}


