﻿using System;
using System.ServiceModel.DomainServices.Client.ApplicationServices;
using System.Windows;

namespace WP7Host
{
    public partial class App : Application
    {

        public App()
        {
            this.Startup += this.Application_Startup;
            this.Exit += this.Application_Exit;
            this.UnhandledException += this.Application_UnhandledException;

            InitializeComponent();
            
            //웹컨텍스트를 이용해서 인증을 한다.
            WebContext webContext = new WebContext();
            //폼인증 방식 사용
            webContext.Authentication = new FormsAuthentication();
            this.ApplicationLifetimeObjects.Add(webContext);
        }

        private void Application_Startup(object sender, StartupEventArgs e)
        {
            //웹컨텍스트를 리소스로 추가
            this.Resources.Add("WebContext", WebContext.Current);
            //자동으로 로그인 - 이 페이지에 다시 들어왔을 경우 로그인 유저가 있으면 로드함
            WebContext.Current.Authentication.LoadUser(this.Application_UserLoaded, null); 

            this.RootVisual = new MainPage();
        }

        private void Application_UserLoaded(LoadUserOperation operation)
        {
            if (operation.HasError == false)
            {}
            else
            {}
        }

        private void Application_Exit(object sender, EventArgs e)
        {

        }

        private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            // If the app is running outside of the debugger then report the exception using
            // the browser's exception mechanism. On IE this will display it a yellow alert 
            // icon in the status bar and Firefox will display a script error.
            if (!System.Diagnostics.Debugger.IsAttached)
            {

                // NOTE: This will allow the application to continue running after an exception has been thrown
                // but not handled. 
                // For production applications this error handling should be replaced with something that will 
                // report the error to the website and stop the application.
                e.Handled = true;
                //예외 처리
                switch (e.ExceptionObject.GetType().ToString())
                {
                    case "DomainOperationException":
                    case "DomainException":
                        break;
                    default:
                        Deployment.Current.Dispatcher.BeginInvoke(delegate { ReportErrorToDOM(e); });
                        break;
                }
            }
        }

        private void ReportErrorToDOM(ApplicationUnhandledExceptionEventArgs e)
        {
            try
            {
                string errorMsg = e.ExceptionObject.Message + e.ExceptionObject.StackTrace;
                errorMsg = errorMsg.Replace('"', '\'').Replace("\r\n", @"\n");

                System.Windows.Browser.HtmlPage.Window.Eval("throw new Error(\"Unhandled Error in Silverlight Application " + errorMsg + "\");");
            }
            catch (Exception)
            {
            }
        }
    }
}
