﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Reactive.Concurrency;
using System.Reactive.Linq;
using System.ServiceModel.DomainServices.Client;
using System.Threading;
using System.Windows;
using WP7Host.Web;
using WP7Host.Web.Services;

namespace WP7Host.Models
{
    public class WP7Model : INotifyPropertyChanged
    {
        /// <summary>
        /// PropertyChanged Event
        /// </summary>
        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected void FirePropertyChanged(string PropertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                if (Deployment.Current.Dispatcher != null &&
                    !Deployment.Current.Dispatcher.CheckAccess())
                {
                    Deployment.Current.Dispatcher.BeginInvoke
                        (
                            (ThreadStart)(
                            () => handler(this, new PropertyChangedEventArgs(PropertyName))
                            )
                        );
                }
                else
                {
                    handler(this, new PropertyChangedEventArgs(PropertyName));
                }
            }
        }

        #endregion

        //도메인 서비스 컨텍스트
        private WP7DomainContext context { get; set; }

        private ObservableCollection<WP7UriList> wp7UriCollection;
        public ObservableCollection<WP7UriList> WP7UriCollection
        {
            get { return wp7UriCollection; }
            set 
            { 
                wp7UriCollection = value;
                FirePropertyChanged("WP7UriCollection");
            }
        }

        private ObservableCollection<WP7History> wp7HistoryCollection;
        public ObservableCollection<WP7History> WP7HistoryCollection
        {
            get { return wp7HistoryCollection; }
            set
            {
                wp7HistoryCollection = value;
                FirePropertyChanged("WP7HistoryCollection");
            }
        }

        public IObservable<WP7UriList> WP7UriObservable { get; set; }

        private string conditionName;

        public string ConditionName
        {
            get { return conditionName; }
            set 
            { 
                conditionName = value;
                FirePropertyChanged("ConditionName");
            }
        }


        private object lastExecuteSender;
        /// <summary>
        /// 마지막 실행 쿼리를 호출한 오브젝트
        /// 메소드를 호출한 정확한 넘을 남겨 놓기 위해 사용
        /// </summary>
        public object LastExecuteSender
        {
            get { return lastExecuteSender;}
            set
            {
                lastExecuteSender = value;
                FirePropertyChanged("LastExecuteSender");
            }
        }

        private string messageData;
        /// <summary>
        /// 메시지 데이터는 쓰임에 따라 Set 방식이 달라지므로 가상 속성을 사용
        /// </summary>
        public string MessageData
        {
            get { return messageData; }
            set
            {
                messageData = value;
                FirePropertyChanged("MessageData");
            }
        }

        public WP7Model()
        {
            context = new WP7DomainContext();

            ConditionName = string.Empty;
            //RxRegist();
        }

        private LoadOperation<WP7UriList> CreateLoad(EntityQuery<WP7UriList> q)
        {
            return context.Load(q);
        }

        public void RxRegist()
        {
            var observCondi = Observable.FromEventPattern<PropertyChangedEventArgs>(this, "PropertyChanged")
                                        .Where(e => e.EventArgs.PropertyName == "ConditionName")
                                        .Select(e => (string)e.Sender)
                                        .DistinctUntilChanged();

            //var queryStream = from input in observCondi
            //                  select context.GetWP7UriListSetQuery()

            var query = from wp in context.GetWP7UriListSetQuery()
                        where (observCondi == null || wp.UserName.Contains(observCondi.FirstOrDefault()))
                        orderby wp.UserName
                        select wp;

            //Windows Phone Reactive API
            //var subscription = Observable.FromEvent<TextChangedEventArgs>(txt, "TextChanged"); 
            var load = CreateLoad(query);

            var loadStream = Observable.FromEventPattern<EventArgs>(load, "Completed");

            loadStream
                .ObserveOn(Scheduler.ThreadPool)
                .ObserveOnDispatcher()
                .Subscribe(result =>
            {
                var complatedOperation = (LoadOperation)result.Sender;
                //LastExecuteSender = complatedOperation.UserState;

                if (complatedOperation.HasError != true)
                {
                    if (complatedOperation.Entities.Count() > 0)
                    {
                        //반환된 데이터 타입을 가지고 데이터 구분해서 모델에 있는 컬렉션에 입력
                        switch (complatedOperation.Entities.FirstOrDefault().GetType().Name)
                        {
                            case "WP7UriList":
                                WP7UriCollection = new ObservableCollection<WP7UriList>(complatedOperation.Entities.Cast<WP7UriList>());
                                break;
                            case "WP7History":
                                WP7HistoryCollection = new ObservableCollection<WP7History>(complatedOperation.Entities.Cast<WP7History>());
                                break;
                        }
                        MessageData = "조회 작업이 성공 했습니다.";
                    }
                    else
                    {
                        MessageData = "조회 결과가 없습니다.";
                    }
                }
                else
                {
                    MessageData = complatedOperation.Error.Message;
                    complatedOperation.MarkErrorAsHandled();
                }
            });
        }

        public void Adding(object addData)
        {
            if (addData != null && addData is WP7UriList)
            {
                WP7UriList addItem = addData as WP7UriList;

                WP7UriCollection.Insert(0, addItem);
                context.WP7UriLists.Add(addItem);
            }
        }

        public void Canceling()
        {
            context.RejectChanges();
            WP7UriCollection = new ObservableCollection<WP7UriList>(context.WP7UriLists);

            MessageData = "취소 작업을 완료 했습니다.";
        }

        /// <summary>
        /// 조건에 조회 하기
        /// 클라이언트에서 쿼리를 만들어서 서버에서 실행하고 결과 받음
        /// </summary>
        public void Getting(string condition, object sender)
        {
            ConditionName = condition;
            //LastExecuteSender = sender;

            //client-side query
            //EntityQuery<WP7UriList> query = from wp in context.GetWP7UriListSetQuery()
            //                                where (condition == null || wp.UserName.Contains(condition))
            //                                orderby wp.UserName
            //                                select wp;

            EntityQuery<WP7UriList> query = from wp in context.GetWP7UriListSetQuery()
                                            where (condition == null || condition == "" || wp.UserName.Contains(condition))
                                            orderby wp.UserName
                                            select wp;
            //만들어진 쿼리를 이용해서 조회
            Scheduler.Schedule(Scheduler.ThreadPool, () => 
            {
                if (context.IsLoading == true)
                {
                    MessageData = "로딩 작업 중입니다.";
                    return;
                }

                context.Load(query,
                            LoadBehavior.RefreshCurrent,
                            load =>
                            {
                                LoadComplet(load);
                            }, sender);
            });

            
            //기본 쿼리 조회
            //context.Load(context.GetWP7UriListSetQuery(),
            //            LoadBehavior.RefreshCurrent,
            //            load =>
            //            {
            //                LoadComplet(load);
            //            }, sender);
        }

        /// <summary>
        /// 결과 처리
        /// </summary>
        /// <param name="load"></param>
        private void LoadComplet(LoadOperation load)
        {
            //마지막 호출한 오브젝트
            LastExecuteSender = load.UserState;

            if (load.HasError != true)
            {
                if (load.Entities.Count() > 0)
                {
                    //반환된 데이터 타입을 가지고 데이터 구분해서 모델에 있는 컬렉션에 입력
                    switch (load.Entities.FirstOrDefault().GetType().Name)
                    {
                        case "WP7UriList":
                            WP7UriCollection = new ObservableCollection<WP7UriList>(load.Entities.Cast<WP7UriList>());
                            break;
                        case "WP7History":
                            WP7HistoryCollection = new ObservableCollection<WP7History>(load.Entities.Cast<WP7History>());
                            break;
                    }
                    MessageData = "조회 작업이 성공 했습니다.";
                }
                else
                {
                    MessageData = "조회 결과가 없습니다.";
                }
            }
            else
            {
                MessageData = "조회 작업이 실패 했습니다.";
            }
        }

        /// <summary>
        /// 삭제
        /// </summary>
        /// <param name="removeData">삭제할 데이터</param>
        public void Removing(object removeData)
        {
            // 저장 안된 값 오류
            if (removeData != null)
            {
                WP7UriList removedata = removeData as WP7UriList;

                switch (removedata.EntityState)
                { 
                    case EntityState.Unmodified:
                        context.WP7UriLists.Remove(removedata);
                        break;
                }
                //WP7UriCollection.Remove(removedata);
            }
        }

        public void Saving()
        {
            if (context.IsSubmitting)
            {
                MessageData = "저장 작업이 진행 중입니다.";
                return;
            }
            else
            {
                context.SubmitChanges(submit =>
                {
                    if (submit.IsComplete == true && submit.HasError ==false)
                    {
                        if (submit.ChangeSet.AddedEntities.Count > 0)
                        {
                            MessageData = "추가 저장 작업을 성공 했습니다.";
                        }
                        if (submit.ChangeSet.ModifiedEntities.Count > 0)
                        {
                            MessageData = "수정 저장 작업을 성공 했습니다.";
                        }
                        if (submit.ChangeSet.RemovedEntities.Count > 0)
                        {
                            submit.ChangeSet.RemovedEntities
                                .Cast<WP7UriList>()
                                .ToList().ForEach(p => WP7UriCollection.Remove(p));
                            MessageData = "삭제 저장 작업을 성공 했습니다.";
                        }
                    }
                    else
                    {
                        MessageData = "저장 작업을 실패 했습니다.\n" + submit.Error.Message;
                        submit.MarkErrorAsHandled();
                        //MessageBox.Show(MessageData, "알림", MessageBoxButton.OK);
                    }
                }, null);
            }
        }
    }
}
