﻿using System;
using System.ComponentModel;
using System.Reactive.Concurrency;
using System.Reactive.Linq;
using System.ServiceModel.DomainServices.Client.ApplicationServices;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Microsoft.Expression.Interactivity.Core;
using WP7Host.Models;
using WP7Host.Views;
using WP7Host.Web;

namespace WP7Host.ViewModels
{
    /// <summary>
    /// Administrator  mvc4dmin
    /// </summary>
    public class MainPageViewModel : INotifyPropertyChanged
    {
        /// <summary>
        /// PropertyChanged Event
        /// </summary>
        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected void FirePropertyChanged(string PropertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                if (Deployment.Current.Dispatcher != null &&
                    !Deployment.Current.Dispatcher.CheckAccess())
                {
                    Deployment.Current.Dispatcher.BeginInvoke
                        (
                            (ThreadStart)(
                            () => handler(this, new PropertyChangedEventArgs(PropertyName))
                            )
                        );
                }
                else
                {
                    handler(this, new PropertyChangedEventArgs(PropertyName));
                }
            }
        }

        #endregion

        ChildWindow popup;

        //모델
        public WP7Model Model { get; set; }

        private string messageData;
        /// <summary>
        /// 메세지 데이터
        /// </summary>
        public string MessageData
        {
            get { return messageData; }
            set
            {
                if (messageData != value && value != null)
                {
                    messageData = value;
                    FirePropertyChanged("MessageData");
                }
            }
        }

        private string condition;
        /// <summary>
        /// 채널 목록 검색 조건
        /// </summary>
        public string Condition
        {
            get { return condition; }
            set 
            { 
                condition = value;
                FirePropertyChanged("Condition");
            }
        }

        private string userName;
        /// <summary>
        /// 채널등록 유저 이름
        /// </summary>
        public string UserName
        {
            get { return userName; }
            set 
            { 
                userName = value;
                FirePropertyChanged("UserName");
            }
        }

        private string userId;
        /// <summary>
        /// 인증 사용자 아이디
        /// </summary>
        public string UserId
        {
            get { return userId; }
            set 
            { 
                userId = value;
                FirePropertyChanged("UserId");
            }
        }

        private string userPW;
        /// <summary>
        /// 인증 사용자 비밀번호
        /// </summary>
        public string UserPW
        {
            get { return userPW; }
            set
            {
                userPW = value;
                FirePropertyChanged("UserPW");
            }
        }

        private bool isLogin;
        /// <summary>
        /// 로그인 여부
        /// </summary>
        public bool IsLogin
        {
            get { return isLogin; }
            set 
            { 
                isLogin = value;
                FirePropertyChanged("IsLogin");
            }
        }


        //생성자
        public MainPageViewModel()
        {
            Model = new WP7Model();

            if (!DesignerProperties.IsInDesignTool)
            {
                Observable.FromEventPattern<PropertyChangedEventArgs>(Model, "PropertyChanged")
                          .ObserveOn(Scheduler.ThreadPool)
                          .ObserveOnDispatcher()
                          .Subscribe(prop =>
                          {
                              switch (prop.EventArgs.PropertyName)
                              {
                                  case "MessageData":
                                      this.MessageData = ((WP7Model)prop.Sender).MessageData;
                                      break;
                              }
                          });
                //로그인 완료 이벤트 연결
                WebContext.Current.Authentication.LoggedIn += new EventHandler<AuthenticationEventArgs>(Authentication_LoggedIn);
                //로그아웃 완료 이벤트 연결
                WebContext.Current.Authentication.LoggedOut += new EventHandler<AuthenticationEventArgs>(Authentication_LoggedOut);
            }

            IsLogin = false;
        }

        void Authentication_LoggedOut(object sender, System.ServiceModel.DomainServices.Client.ApplicationServices.AuthenticationEventArgs e)
        {
            Display();
        }

        void Authentication_LoggedIn(object sender, System.ServiceModel.DomainServices.Client.ApplicationServices.AuthenticationEventArgs e)
        {
            Display();
        }

        private ICommand loginCommand;
        /// <summary>
        /// 로그인 커맨드
        /// </summary>
        public ICommand LoginCommand
        {
            get
            {
                if (loginCommand == null)
                {
                    loginCommand = new ActionCommand(() =>
                    {
                        // 파라미터로 넘겨줄 아이디와 패스워드
                        LoginParameters lp = new LoginParameters(UserId, UserPW, false, null);

                        // 로그인 처리 - 로그인이 완료되면 위에 만들어 놓은 로그인 완료 이벤트 실행됨
                        WebContext.Current.Authentication.Login(lp, load =>
                        {
                            if (load.HasError == false && load.LoginSuccess == true)
                            {
                            }
                            else
                            {
                                MessageBox.Show("로그인 실패");
                            }
                        }, null);                        
                    }
                    );
                }
                return loginCommand;
            }
        }

        private ICommand logoutCommand;
        /// <summary>
        /// 로그아웃 커맨드
        /// </summary>
        public ICommand LogoutCommand
        {
            get
            {
                if (logoutCommand == null)
                {
                    logoutCommand = new ActionCommand(() =>
                    {
                        WebContext.Current.Authentication.Logout(load =>
                        {
                            //Display();
                        }, null);
                    }
                    );
                }
                return logoutCommand;
            }
        }

        private void Display()
        {
            if (WebContext.Current.User.IsAuthenticated == true)
            {
                IsLogin = true;
                UserName = WebContext.Current.User.DisplayName;
                MessageData = "로그인에 성공 했습니다.";
            }
            else
            {
                IsLogin = false;
                MessageData = "로그인이 필요 합니다.";
            }
        }

        private ICommand addCommand;
        /// <summary>
        /// 추가 커맨드
        /// </summary>
        public ICommand AddCommand
        {
            get
            {
                if (addCommand == null)
                {
                    addCommand = new ActionCommand(() =>
                    {
                        WP7UriList user = new WP7UriList 
                        {
                            UserName = UserName,
                            ChannelUri = "test",
                            RegDT = DateTime.Now,
                            UptDT = DateTime.Now,
                            UseBT = true
                        };
                        Model.Adding(user);
                    }
                    );
                }
                return addCommand;
            }
        }

        private ICommand cancelCommand;
        /// <summary>
        /// 취소 커맨드
        /// </summary>
        public ICommand CancelCommand
        {
            get
            {
                if (cancelCommand == null)
                {
                    cancelCommand = new ActionCommand(() =>
                    {
                        Model.Canceling();
                    });
                }
                return cancelCommand;
            }
        }

        private ICommand removeCommand;
        /// <summary>
        /// 삭제 커맨드
        /// </summary>
        public ICommand RemoveCommand
        {
            get
            {
                if (removeCommand == null)
                {
                    removeCommand = new ActionCommand(item =>
                    {
                        WP7UriList wp7uri = item as WP7UriList;
                        var result = MessageBox.Show(wp7uri.UserName + " 채널을 삭제 하시겠습니까?", "확인", MessageBoxButton.OKCancel);
                        if (result == MessageBoxResult.OK)
                        {
                            //wp7uri.WP7History.ToList().ForEach(p => wp7uri.WP7History.Remove(p));
                            Model.Removing(wp7uri);
                            Model.Saving();
                        }
                    });
                }
                return removeCommand;
            }
        }

        private ICommand saveCommand;
        /// <summary>
        /// 저장 커맨드
        /// </summary>
        public ICommand SaveCommand
        {
            get
            {
                if (saveCommand == null)
                {
                    saveCommand = new ActionCommand(() =>
                    {
                        Model.Saving();
                    });
                }
                return saveCommand;
            }
        }

        private ICommand selectCommand;
        /// <summary>
        /// 조회 커맨드 
        /// </summary>
        public ICommand SelectCommand
        {
            get
            {
                if (selectCommand == null)
                {
                    selectCommand = new ActionCommand(() =>
                    {
                        //Model.RxRegist();
                        //조회 함수 호출
                        Model.Getting(Condition, this);
                    });
                }
                return selectCommand;
            }
        }

        private ICommand testCommand;
        /// <summary>
        /// Test 커맨드 
        /// </summary>
        public ICommand TestCommand
        {
            get
            {
                if (testCommand == null)
                {
                    testCommand = new ActionCommand(() =>
                    {
                        //팝업 차일드 윈도우 출력
                        popup.Show();
                    });
                }
                return testCommand;
            }
        }

        private ICommand loadedCommand;
        /// <summary>
        /// 뷰 로드 이벤트에서 실행되는 커맨드
        /// </summary>
        public ICommand LoadedCommand
        {
            get
            {
                if (loadedCommand == null)
                {
                    loadedCommand = new ActionCommand(() =>
                    {
                        popup = new ChildWindow { Content = new SilverlightControl1() };
                    });
                }
                return loadedCommand;
            }
        }
    }
}
